import Globals, OFS, AccessControl   #, os

from AccessControl import ClassSecurityInfo
from Acquisition import Implicit   #, aq_parent
from Globals import HTMLFile, Persistent
from DateTime import DateTime

manage_addZUBBMessageForm = HTMLFile('dtml/ZUBBMessage_addForm', globals())

class ZUBBMessage(OFS.SimpleItem.SimpleItem,
             OFS.PropertyManager.PropertyManager,
             Persistent,
             Implicit,
             AccessControl.Role.RoleManager):
    """ZUBBMessage object"""

    meta_type = "ZUBB Message"

    icon = 'misc_/ZUBB/zubb_message.gif'

    _properties = (
        {'id':'title', 'type':'string', 'mode':'w'},
        {'id':'author', 'type':'string', 'mode':'w'},
        {'id':'email', 'type':'string', 'mode':'w'},
        {'id':'body', 'type':'text', 'mode':'w'},
        {'id':'in_reply_to', 'type':'string', 'mode':''},
        {'id':'priority', 'type':'integer', 'mode':'w'}
        )

    manage_options=(
        {'label': 'Contents',
         'action': 'manage_main'},
        
        {'label': 'View',
         'action': ''},
        
        {'label': 'Security',
         'action': 'manage_access'}
        )


    security = ClassSecurityInfo()

    security.declareProtected('Manage ZUBB Folders', 'manage_addProperty',
                                                    'manage_editProperties',
                                                    'manage_delProperties',
                                                    'manage_changeProperties')

    def __init__(self, id, title, body, author, email, in_reply_to, priority = 0):
        self.id = id
        self.title = title.strip()
        self.author = author.strip()
        self.email = email.strip()
        self.body = body
        self.in_reply_to = in_reply_to
        self.priority = int(priority)
        self.posted_date = self.last_post_date = DateTime()
        self.last_post_author = ''

    security.declareProtected('Manage ZUBB Folders', 'manage_main')
    manage_main = HTMLFile('dtml/ZUBBMessage_editForm', globals())   # Management Interface

    def manage_afterAdd(self, item, container):
        self.no_orphan_fixing = 0   # allow thread flattening
        OFS.SimpleItem.SimpleItem.manage_afterAdd(self, item, container)
        self._update_thread(self)   # <-- catalog done in here

    def manage_beforeDelete(self, item, container):
        self.no_orphan_fixing = 1   # so that we wont flatten our thread if this is a move / rename
        parent = self.aq_parent
        id = self.getId()
        self.uncatalog_object(id)
        if not getattr(parent,'no_orphan_fixing',0):
            # Prevent orphans
            for child in self.children():
                child.in_reply_to = self.in_reply_to
                child._catalog()
            # Set new last post for parent if we need to
            parent = getattr(self.aq_parent, self.in_reply_to)
            if parent.last_post_date == self.posted_date:
                parent._update_thread(parent.thread_last_post(id))   # find newest message (ignoring current one)
        OFS.SimpleItem.SimpleItem.manage_beforeDelete(self, item, container)

    security.declareProtected('Manage ZUBB Folders', 'manage_edit')
    def manage_edit(self, REQUEST):
        """Post a ZUBB Message"""
        f = REQUEST.form
        title = f.get('title', '')
        body = f.get('body', '')
        in_reply_to = f.get('in_reply_to', '')
        priority = f.get('priority', '')
        ok = 0
        if in_reply_to:
            if not hasattr(self.aq_parent.aq_explicit, in_reply_to):
                msg = 'Not updated: "In Reply To" ID did not exist.'
            elif in_reply_to == self.id:
                msg = 'Not updated: tried to reference self for "In Reply To".'
            else:
                ok = 1
        else:
            ok = 1
        if ok and (title or body):
            self.title = title
            self.body = body
            self.author = f.get('author', '')
            self.email = f.get('email', '')
            self.in_reply_to = in_reply_to
            if priority:
                self.priority = 1
            else:
                self.priority = 0
            self._catalog()
            msg = "ZUBB message updated."
        if REQUEST is not None:
            return self.manage_main(self, REQUEST, manage_tabs_message = msg)

    security.declarePrivate('_catalog')
    def _catalog(self):
        """ Catalog this message """
        self.catalog_object(self, self.getId())

    security.declarePrivate('_update_thread')
    def _update_thread(self, last_post):
        """ Changes the last post down the family tree """
        last_post_date = last_post.posted_date
        last_post_author = last_post.author
        self.last_post_date = last_post_date
        self.last_post_author = last_post_author
        self._catalog()
        irp = self.in_reply_to
        if irp:
            getattr(self.aq_parent, irp)._update_thread(last_post)

    security.declarePublic('index_html')
    def index_html(self):
        """ standard_view """
        if hasattr(self.aq_parent, 'zubb_options'):
            zubb_options = self.aq_parent.zubb_options()
        else:
            zubb_options = {}
        # default view to the primary_view
        view = zubb_options.get('primary_view', 1)
        req = self.REQUEST
        if req:
            # see if 'view' is a form variable
            if req.form.has_key('view'):
                view = int(req.form['view'])
                # set a cookie
                req.RESPONSE.setCookie('zubbView', view, path='/')
            # otherwise try getting 'view' from cookies
            elif req.cookies.has_key('zubbView'):
                view = int(req.cookies['zubbView'])
        if view == 2:
            return self.message_inline(None, self)
        else:
            return self.message_indent(None, self)

    security.declarePublic('all_text')
    def all_text(self):
        """ Returns the title and the body text - used to index for search """
        return '%s %s' % (self.title, self.body)

    security.declarePublic('ancestors')
    def ancestors(self):
        """Return a list of ancestral posts"""
        ancestors = []
        irp = self.in_reply_to
        folder = self.aq_parent.aq_explicit
        while irp:
            parent = getattr(folder, irp)
            ancestors.append(parent)
            irp = parent.in_reply_to
        return ancestors

    security.declarePublic('children')
    def children(self):
        """ Return a list of replies to this message - most recently posted first """
        children = []
        results = self.searchResults(in_reply_to=self.id, sort_on='last_post_date', sort_order='reverse')
        children = map(lambda x: x.getObject(), results)
        return children

    security.declarePublic('child_last_post')
    def child_last_post(self, ignore_deleted=''):
        """ Looks through children and returns the newest message"""
        last_post_date = self.posted_date
        last_post = self
        for message in self.children():
            if message.id != ignore_deleted:
                test_post = message
                test_post_date = message.child_last_post()
                if test_date > last_post_date:
                    last_post = test_post
                    last_post_date = test_post_date
        return last_post

    security.declarePublic('childMessageCount')
    def childMessageCount(self):
        """ Counts the number of children of this message, including this message"""
        count = 1   # this message
        for message in self.children():
                count = count + message.childMessageCount()
        return count

    security.declarePublic('descendants')
    def descendants(self):
        """ Return a list of descendants and the date they were posted (including self) """
        # add self
        descendants = [(self.posted_date, self)]
        # add children
        for child in self.children():
            descendants = descendants + child.descendants()
        return descendants

    security.declarePublic('indent')
    def indent(self):
        """ Return all the descendant posts tree style """
        self_irp = self.in_reply_to
        thread = []
        threadidx = []
        index = -1
        for descendantdate, descendant in self.descendants():
            index = index + 1
            irp = descendant.in_reply_to
            if irp == self_irp:
                reply = -1
                indent = 0
            else:
                reply = threadidx.index(irp)
                indent = thread[reply][1] + 1
            thread.append((descendant, indent))
            threadidx.append(descendant.getId())
        return thread

    security.declarePublic('inline')
    def inline(self):
        """ Return all the descendant posts inline (flatfile style) """
        descendants = self.descendants()
        descendants.sort()   # now it's in order by posted date
        self_irp = self.in_reply_to
        thread = []
        threadidx = []
        index = -1
        for descendantdate, descendant in descendants:
            index = index + 1
            irp = descendant.in_reply_to
            if irp == self_irp:
                reply = -1
            else:
                reply = threadidx.index(irp)
            thread.append((descendant, index, reply))
            threadidx.append(descendant.getId())
        return thread

    security.declarePublic('threadMessageCount')
    def threadMessageCount(self):
        """ Looks at the topmost ancestor, and then counts children"""
        message_count = self.thread_top().childMessageCount()
        return message_count

    security.declarePublic('thread_last_post')
    def thread_last_post(self, ignore_deleted=''):
        """ Looks at the topmost ancestor, and then recurses
            through the messages finding the last posted message"""
        last_post = self.thread_top().child_last_post(ignore_deleted)
        return last_post

    security.declarePublic('thread_top')
    def thread_top(self):
        """ returns the top ancestor """
        top = self
        irp = top.in_reply_to
        folder = self.aq_parent.aq_explicit
        while irp:
            top = getattr(folder, irp)
            irp = top.in_reply_to
        return top


# This gets the security assertions going
Globals.InitializeClass(ZUBBMessage)
